#!/usr/bin/perl
# j5_xml_rpc_client_condense_multiple_assembly_files.pl
# Nathan Hillson 01-SEP-15

# *** Copyright Notice ***
#
# Copyright (c) 2010-2015, The Regents of the University of California,
# through Lawrence Berkeley National Laboratory (subject to receipt of
# any required approvals from the U.S. Dept. of Energy). All rights
# reserved.
#
# NOTICE. This software was developed under funding from the U.S. Department of Energy.
# As such, the U.S. Government has been granted for itself and others acting on its
# behalf a paid-up, nonexclusive, irrevocable, worldwide license in the Software to
# reproduce, prepare derivative works, and perform publicly and display publicly.
# Beginning five (5) years after the date permission to assert copyright is obtained
# from the U.S. Department of Energy, and subject to any subsequent five (5) year
# renewals, the U.S. Government is granted for itself and others acting on its behalf a
# paid-up, nonexclusive, irrevocable, worldwide license in the Software to reproduce,
# prepare derivative works, distribute copies to the public, perform publicly and
# display publicly, and to permit others to do so. 
#
# This program is free software; you can redistribute it and/or modify
# it under the same terms as the Perl "Artistic License"
# (http://dev.perl.org/licenses/artistic.html).
#
# *** End of Copyright Notice ***

use XML::RPC;
use MIME::Base64;
require LWP::UserAgent;

my $assembly_files_to_condense_filename;
my $assembly_method;
my $encoded_assembly_files_to_condense_file;
my $encoded_zipped_assembly_files_file;
my $input_file_handle;
my $j5_session_id;
my $output_file_handle;
my $output_filename;
my $result;
my $ua;
my $xmlrpc;
my $zipped_assembly_files_filename;

if (@ARGV != 3) {
    die "Usage: $0 j5_session_id AssemblyFilesToCondenseFileName ZippedAssemblyFilesFileName\n";
}

$j5_session_id = $ARGV[0];
$assembly_files_to_condense_filename = $ARGV[1];
$zipped_assembly_files_filename = $ARGV[2];

# encode the assembly_files_to_condense_file
open($input_file_handle, "$assembly_files_to_condense_filename") or die "$!";
while (read($input_file_handle, $input_buffer, 60*57)) {
  $encoded_assembly_files_to_condense_file.=encode_base64($input_buffer);
}
chomp $encoded_assembly_files_to_condense_file;
close($input_file_handle);

# encode the zipped_assembly_files_file
open($input_file_handle, "$zipped_assembly_files_filename") or die "$!";
while (read($input_file_handle, $input_buffer, 60*57)) {
  $encoded_zipped_assembly_files_file.=encode_base64($input_buffer);
}
chomp $encoded_zipped_assembly_files_file;
close($input_file_handle);

# setting the user agent timeout to a longer period of time is required to make sure that j5 has sufficient time to complete an assembly
$ua = LWP::UserAgent->new();
$ua->timeout( 86400 ); # wait up until 24 hours for j5 to complete assembly, then bail
$xmlrpc = XML::RPC->new('https://j5.jbei.org/bin/j5_xml_rpc.pl',lwp_useragent => $ua);

# call the j5 xml-rpc service to condense the assembly files
$result = $xmlrpc->call( 'CondenseMultipleAssemblyFiles', { 
							   j5_session_id => $j5_session_id,
							   encoded_assembly_files_to_condense_file => $encoded_assembly_files_to_condense_file,
							   encoded_zipped_assembly_files_file => $encoded_zipped_assembly_files_file,
							  }
		       );

if (defined $result->{"faultString"}) {
  print "Error: faultCode: ",$result->{"faultCode"}," faultString: ",$result->{"faultString"},"\n";
}

else {

  if (defined $result->{"error_message"}) {
    print "Error: ",$result->{"error_message"},"\n";
  }

  $output_filename = $result->{"output_filename"};
  open($output_file_handle, ">$output_filename") or die "$!";
  print $output_file_handle decode_base64($result->{"encoded_output_file"});
  close($output_file_handle);
}
